import QtQuick 2.0
import Sailfish.Silica 1.0
import harbour.metacrane.MetacraneClient 1.0
import QtQuick.LocalStorage 2.0 as Sql

Page {
    id: page
    allowedOrientations: Orientation.All

    Label {
        anchors.top: page.top
        anchors.right: page.right
        text: MetacraneClient.uploadProgress < 0 && MetacraneClient.queueSize == 0 ?
            "" :
            "Uploading... "+Math.floor(MetacraneClient.uploadProgress*100)+"%"+(
                MetacraneClient.queueSize > 0 ?
                    " ("+MetacraneClient.queueSize+")" : "")
    }

    SilicaListView {
        id: listview
        anchors.fill: parent

        PullDownMenu {
            MenuItem {
                text: "About"
                onClicked: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
            }
            MenuItem {
                text: "Clear History"
                onClicked: {
                    var db = getDB()
                    db.transaction(function(tx){
                        tx.executeSql("DELETE FROM crane")
                    })
                    refreshList()
                }
            }
        }

        header: Component { Column {
                id: column

                //width: page.width
                spacing: Theme.paddingLarge
                PageHeader {
                    title: "Metacrane"
                }
                TextField {
                    id: crane_name
                    focus: true
                    text: ""
                    placeholderText: "Crane Name"
                    width: page.width
                    height: Theme.itemSizeLarge
                    EnterKey.enabled: text.length > 0
                    EnterKey.iconSource: "image://theme/icon-m-enter-accept"
                    EnterKey.onClicked: {
                        var crane_name = text
                        text = ""
                        MetacraneClient.gotoCrane(crane_name)
                        pageStack.push(Qt.resolvedUrl("CranePage.qml"), {
                            crane_name: crane_name
                        })

                        var db = getDB()
                        db.transaction(function(tx){
                            tx.executeSql("DELETE FROM crane"+
                                    " WHERE name=?", [crane_name])
                        })
                        db.transaction(function(tx){
                            tx.executeSql("INSERT INTO crane(name)"+
                                    " VALUES(?)", [crane_name])
                        })

                        refreshList()
                    }
                    onClicked: {
                        //console.log("crane_name.onClicked")
                        MetacraneClient.pokeConnection()
                    }
                    onTextChanged: {
                        //console.log("crane_name.onTextChanged")
                        MetacraneClient.pokeConnection()
                    }
                }
            }
        }

        VerticalScrollDecorator {}
        model: craneList
        delegate: ListItem {
            id: listitem
            height: Theme.itemSizeSmall
            Label {
                text: name
                anchors.verticalCenter: parent.verticalCenter
                //anchors.horizontalCenter: parent.horizontalCenter
                //anchors.left: parent.left + Theme.paddingLarge
                //anchors.margins: Theme.paddingLarge
                //x: Theme.paddingLarge
                anchors.left: parent.left
                anchors.margins: Theme.paddingLarge
                color: listitem.highlighted ?
                        Theme.highlightColor : Theme.primaryColor
            }
            onClicked: {
                console.log("listitem.onClicked")
                var crane_name = name
                MetacraneClient.gotoCrane(crane_name)
                pageStack.push(Qt.resolvedUrl("CranePage.qml"), {
                    crane_name: crane_name
                })

                var db = getDB()
                db.transaction(function(tx){
                    tx.executeSql("DELETE FROM crane"+
                                  " WHERE name=?", [crane_name])
                })
                db.transaction(function(tx){
                    tx.executeSql("INSERT INTO crane(name)"+
                                  " VALUES(?)", [crane_name])
                })
                // Don't refresh the list because it is annoying
                //refreshList()
            }
        }

        ListModel {
            id: craneList
            Component.onCompleted: {
                refreshList()
            }
        }
    }

    onEntered: {
        console.log("FirstPage.onEntered")
        MetacraneClient.pokeConnection()
    }

    Connections {
        target: MetacraneClient
        onLastMessageChanged: {
            console.log("onLastMessageChanged: "+MetacraneClient.lastMessage)
            showMessageTimer.start()
        }
    }
    Timer {
        id: showMessageTimer
        interval: 1000
        onTriggered: {
            pageStack.push("MessageDialog.qml", {
                message: MetacraneClient.lastMessage
            })
        }
    }

    function refreshList(){
        craneList.clear()
        var db = getDB()
        db.transaction(function(tx){
            // NOTE: Not sure whether the ORDER BY actually works
            var rs = tx.executeSql("SELECT * FROM crane"+
                                   " ORDER BY usage_timestamp DESC");
            var craneNames = []
            for(var i = 0; i < rs.rows.length; i++) {
                var row = rs.rows.item(i);
                console.log("* "+row.name)
                craneList.append({name: row.name})
                craneNames.push(row.name)
            }
            MetacraneClient.updateSavedCraneNames(craneNames)
        });
    }

    function getDB(){
        var db = Sql.LocalStorage.openDatabaseSync("metacrane_cranenames", "1",
                "Metacrane: Crane Names", 1000);
        db.transaction(function(tx){
            tx.executeSql("CREATE TABLE IF NOT EXISTS crane("+
                    "id PRIMARY KEY, name TEXT"+
                    ", usage_timestamp INTEGER DEFAULT CURRENT_TIMESTAMP)")
        })
        return db
    }
}
