import QtQuick 2.0
import Sailfish.Silica 1.0
import harbour.metacrane.MetacraneClient 1.0

Dialog {
    allowedOrientations: Orientation.All
    property var files: []
    property double deletionTime: 1.0

    id: page
    canAccept: true

    SilicaListView {
        id: listView
        anchors.fill: parent
        header: Component { Column {
            PageHeader {
                title: "Confirm Upload | " + MetacraneClient.currentCraneName
            }
            Slider {
                id: timeSlider
                minimumValue: realToTimeSlider(10*60)
                maximumValue: 1.0
                value: 1.0
                valueText:
                    timeSliderToReal(value) < 120*60 ?
                        "" + Math.floor(timeSliderToReal(value) / 60) + " minutes" :
                    Math.floor(timeSliderToReal(value) / 3600) + " hours"
                label: "Deletion Time"
                x: Theme.paddingLarge
                width: page.width - Theme.paddingLarge * 2

                onValueChanged: {
                    deletionTime = timeSliderToReal(value)
                }
                Component.onCompleted: {
                    deletionTime = timeSliderToReal(value)
                }
            }
        }}
        footer: Component { Column {
        }}
        model: files
        delegate: ListItem {
            id: delegate
            height: mainLabel.height + Theme.paddingMedium * 2

            Label {
                id: mainLabel
                x: Theme.fontSizeSmall*2.5 + Theme.paddingLarge
                width: page.width - Theme.paddingLarge * 2 - Theme.fontSizeSmall*2.5
                text: modelData
                anchors.top: parent.top
                anchors.margins: Theme.paddingMedium
                wrapMode: Text.Wrap
            }
            Label {
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.margins: Theme.paddingLarge
                font.pixelSize: Theme.fontSizeSmall
                color: Theme.secondaryColor
                text: "File"
            }
        }
        VerticalScrollDecorator {}
    }

    onAccepted: {
        console.log("Accepted with deletion time "+deletionTime)
        MetacraneClient.uploadFiles(files, deletionTime)
    }

    function timeSliderToReal(value){
        return Math.pow(value, 3) * 56*3600
    }
    function realToTimeSlider(seconds){
        return Math.pow(seconds/(56*3600), 1/3)
    }
}
