import QtQuick 2.0
import Sailfish.Silica 1.0
import harbour.metacrane.MetacraneClient 1.0
import harbour.metacrane.FileSearch 1.0

Dialog {
    property string crane_name: "Unknown"
    id: page
    allowedOrientations: Orientation.All

    FileSearch {
        id: fileSearch
    }

    SilicaListView {
        id: listView
        anchors.fill: parent

        PullDownMenu {
            MenuItem {
                text: "Select All"
                onClicked: {
                    for(var i=0; i<fileSearch.result.length; i++){
                        var r = fileSearch.result[i]
                        r.selected = true
                    }
                }
            }
        }

        header: Component { Column {
            id: column
            spacing: Theme.paddingSmall
            PageHeader {
                title: "Pick File"
            }
            /*Label {
                id: pathLabel
                font.pixelSize: Theme.fontSizeSmall
                text: fileSearch.searchPath
            }*/
            SearchField {
                id: searchField
                focus: true
                text: ""
                placeholderText: "Filter"
                width: page.width
                height: Theme.itemSizeLarge
                //EnterKey.enabled: true
                EnterKey.iconSource: "image://theme/icon-m-enter-accept"
                EnterKey.onClicked: {
                    fileSearch.search(text)
                    searchField.focus = false
                }
                onTextChanged: {
                    // Can't use this because modified model steals focus
                    // from keyboard
                    //fileSearch.search(text)
                    if(text === "")
                        fileSearch.search("")
                }
            }
        }}
        footer: Component { Column {
            id: column
            spacing: Theme.paddingSmall
            Label {
                id: pathLabel
                x: Theme.paddingLarge
                font.pixelSize: Theme.fontSizeSmall
                text: fileSearch.searchPath
                color: Theme.secondaryColor
            }
        }}

        model: fileSearch.result
        delegate: ListItem {
            id: delegate
            //height: Theme.itemSizeSmall + Theme.paddingMedium * 2
            //height: Theme.itemSizeSmall
            height:
                mainLabel.height + Theme.paddingMedium * 2 > Theme.itemSizeSmall ?
                    mainLabel.height + Theme.paddingMedium * 2 :
                Theme.itemSizeSmall
            highlighted: modelData.selected

            Label {
                id: mainLabel
                x: Theme.fontSizeSmall*2.5 + Theme.paddingLarge
                width: page.width - Theme.paddingLarge * 2 - Theme.fontSizeSmall*2.5
                text: modelData.name === ".." ? ".. (parent directory)" :
                        modelData.name
                //anchors.verticalCenter: parent.verticalCenter
                anchors.top: parent.top
                anchors.margins: Theme.paddingMedium
                color: delegate.highlighted ? Theme.highlightColor : Theme.primaryColor
                wrapMode: Text.Wrap
                font.underline: modelData.type === "dir"
            }
            Label {
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.margins: Theme.paddingLarge
                font.pixelSize: Theme.fontSizeSmall
                color: Theme.secondaryColor
                text: modelData.type == "file" ? "File" :
                        modelData.type == "dir" ? "Dir" :
                        modelData.type
            }
            Image {
                anchors.top: parent.top
                anchors.left: parent.left
                source:
                    modelData.path.match(/.*\.(jpg|png|JPG|PNG)/) ?
                        modelData.path :
                    ""
                // Size in memory (will be scaled if source is larger)
                sourceSize.height: 128
                sourceSize.width: 128
                // Size on screen
                height: Theme.itemSizeSmall
                width: Theme.itemSizeSmall
            }
            Label {
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                anchors.margins: Theme.paddingLarge
                font.pixelSize: Theme.fontSizeSmall
                color: Theme.secondaryColor
                text: modelData.type === "file" ? prettySize(modelData.size) :
                        ""
            }
            onClicked: {
                console.log("Clicked "+index+": "+modelData.path)
                if(modelData.type == "dir"){
                    fileSearch.searchPath = modelData.path
                    searchField.text = ""
                    return
                }
                modelData.selected = !modelData.selected
            }
        }
        VerticalScrollDecorator {}
    }

    onAccepted: {
        var files = []
        for(var i=0; i<fileSearch.result.length; i++){
            var r = fileSearch.result[i]
            if(r.type === "file" && r.selected)
                files.push(r.path)
        }
        pageStack.replace(Qt.resolvedUrl("UploadFileConfirm.qml"), {
            files: files
        })
    }

    function prettySize(size){
        if(size >= 1024*1024*2)
            return ""+Math.round(size/1024/1024)+"MB"
        else if(size >= 1024*2)
            return ""+Math.round(size/1024)+"KB"
        return ""+size+"B"
    }}
