import QtQuick 2.0
import Sailfish.Silica 1.0
import harbour.metacrane.MetacraneClient 1.0

Page {
    property string item_text
    id: page
    allowedOrientations: Orientation.All

    PageHeader {
        id: pageHeader
        title: "Text Item"
    }
    Label {
        id: linksLabel
        text: "Parsing links..."
        textFormat: Text.StyledText
        anchors.top: pageHeader.bottom
        x: Theme.paddingLarge
        width: page.width - Theme.paddingLarge * 2
        wrapMode: Text.Wrap
        onLinkActivated: {
            if(link.match(/^mc:.*/)){
                MetacraneClient.gotoCrane(link.substring(3))
                pageStack.pop()
                return
            }
            MetacraneClient.openUrl(link)
        }
    }
    TextArea {
        id: textArea
        focus: true
        //anchors.fill: parent
        //anchors.margins: Theme.paddingLarge
        anchors.centerIn: parent
        x: Theme.paddingLarge
        width: page.width - Theme.paddingLarge * 2
        text: item_text
        wrapMode: Text.Wrap
        readOnly: true
    }
    Button {
        anchors.horizontalCenter: page.horizontalCenter
        anchors.bottom: page.bottom
        anchors.margins: Theme.paddingLarge
        text: "Copy All"
        onClicked: {
            Clipboard.text = textArea.text
            pageStack.pop()
        }
    }

    // Parse URL(s) and allow opening them
    Component.onCompleted: {
        console.log("onCompleted")
        updateLinks()
    }
    onItem_textChanged: {
        console.log("onItem_textChanged")
        updateLinks()
    }

    function updateLinks(){
        var formatted_links = ""
        var text = item_text
        var old_text = ""
        // http(s)://<stuff>
        do{
            old_text = text
            text = text.replace(/https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/,
                    function(orig, p1, offset, full_text)
            {
                formatted_links += '<br><a href="'+orig+'">'+orig+'</a>'
                return ''
            })
        }while(text != old_text);
        // mc:<crane>
        do{
            old_text = text
            text = text.replace(/([^a-zA-Z])mc:([a-zA-Z0-9_.*]{3,50})([ ,.!?]?)/,
                    function(orig, pre, mid, end, offset, full_text)
            {
                mid = mid !== undefined ? mid : ""
                formatted_links += '<br><a href="mc:'+mid+'">mc:'+mid+'</a>'
                return ''
            })
        }while(text != old_text);
        // Create label
        if(formatted_links != "")
            linksLabel.text = "Links:"+formatted_links
        else
            linksLabel.text = "No links."
    }
}
