#ifndef METACRANECLIENT_H
#define METACRANECLIENT_H

#include <QObject>
#include <QDebug>
#include <QList>
#include <QDBusInterface>
#include "metacranesocket.h"

class QFile;

class Entity : public QObject
{
    Q_OBJECT
    Q_PROPERTY(int entity READ entity)
    Q_PROPERTY(QVariantMap components READ components NOTIFY componentsChanged)

    int m_entity;
    QVariantMap m_components;
public:
    Entity(int entity): m_entity(entity) {}
    QVariantMap& components(){ return m_components; }
    int entity(){ return m_entity; }
signals:
    void componentsChanged();
};

class MetacraneClient : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString currentCraneName READ currentCraneName NOTIFY currentCraneNameChanged)
    Q_PROPERTY(QList<QObject*> entities READ entities NOTIFY entitiesChanged)
    Q_PROPERTY(QString lastMessage READ lastMessage NOTIFY lastMessageChanged)
    Q_PROPERTY(double uploadProgress READ uploadProgress NOTIFY uploadProgressChanged)
    Q_PROPERTY(int queueSize READ queueSize NOTIFY queueSizeChanged)

    Q_CLASSINFO("D-Bus Interface", "harbour.metacrane.client")

    //QDBusInterface *m_iface;

    MetacraneSocket m_socket;
    QStringList m_saved_crane_names;
    QString m_current_crane_name;
    QList<QObject*> m_entities;
    QString m_last_message;
    int m_next_transfer_id;
    QStringList m_file_queue;
    int m_deletion_time;

    QFile *m_current_file;
    QDataStream *m_current_file_stream;
    int m_current_transfer_id;
    int m_current_transfer_num_pieces;
    int m_current_transfer_piece_index;

public:
    explicit MetacraneClient(QObject *parent = 0);
    ~MetacraneClient();

    Q_INVOKABLE void pokeConnection();
    Q_INVOKABLE void gotoCrane(QString name, std::function<void(bool success)> cb);
    Q_INVOKABLE void gotoCrane(QString name);
    Q_INVOKABLE QString currentSafeId(){ return m_current_crane_name; }
    Q_INVOKABLE void openUrl(QString url);
    Q_INVOKABLE void clearMessage();
    Q_INVOKABLE void uploadText(QString text, int deletion_time);
    Q_INVOKABLE void uploadFiles(QStringList paths, int deletion_time);
    Q_INVOKABLE void updateSavedCraneNames(QStringList crane_names);

    void addMessageHandler(QString name, MetacraneSocket::msg_cb_t handler);
    QString currentCraneName(){ return m_current_crane_name; }
    QList<QObject*>& entities(){ return m_entities; }
    QString lastMessage(){ return m_last_message; }
    double uploadProgress();
    int queueSize();

public slots:
    // TODO: Excplicit mime type
    Q_SCRIPTABLE void sendFile(QString path, QString fileName, QString craneName, int deletionTime);
    Q_SCRIPTABLE QStringList getSavedCraneNames();

signals:
    void currentCraneNameChanged();
    void entitiesChanged();
    void lastMessageChanged();
    void uploadProgressChanged();
    void queueSizeChanged();

protected:
    void addBuiltinHandlers();
    void sendMessage(QString name, const QVariantMap &params);
    void sendRequest(QString name, const QVariantMap &params,
            std::function<void(const QVariantMap &response)> cb);
    void showMessage(QString message);
    void uploadNextFile();
    void sendNextFilePiece();
    void endCurrentTransfer();

protected slots:
    void socketConnected();
    void socketDisconnected();
    void update();
};

#endif // METACRANECLIENT_H
