#ifndef METACRANESOCKET_H
#define METACRANESOCKET_H

#include <QObject>
#include <QDebug>
#include <QList>
#include <QtWebsocket/QWsSocket.h>
#include <functional>

class QTimer;

class MetacraneSocket : public QObject
{
    Q_OBJECT
public:
    typedef std::function<void(const QVariantMap &response)> msg_cb_t;
    typedef std::function<void(const QVariantMap &response)> req_cb_t;
    enum ConnStatus {CS_DISCONNECTED, CS_CONNECTING, CS_CONNECTED};
protected:
    QtWebsocket::QWsSocket *m_ws;
    ConnStatus m_conn_status;
    QTimer *m_conn_status_timer;
    int m_next_req_id;
    QMap<QString, msg_cb_t> m_msg_cbs;
    QMap<int, req_cb_t> m_req_cbs;

public:
    explicit MetacraneSocket(QObject *parent = 0);

    Q_INVOKABLE void pokeConnection();
    Q_INVOKABLE bool isConnected();

    void addMessageHandler(QString name, msg_cb_t handler);
    void sendMessage(QString name, const QVariantMap &params);
    void sendRequest(QString name, const QVariantMap &params,
            std::function<void(const QVariantMap &response)> cb);

public slots:

signals:
    void connected();
    void disconnected();

protected:
    void addBuiltinHandlers();

protected:

protected slots:
    void connectionTimeout();
    void socketConnected();
    void socketDisconnected();
    void handleMessage(QString message);
    void socketStateChanged(QAbstractSocket::SocketState socketState);
    void handleSslErrors(const QList<QSslError>& errors);
};

#endif // METACRANESOCKET_H
