import QtQuick 2.0
import Sailfish.Silica 1.0
import org.nemomobile.dbus 2.0
import Sailfish.TransferEngine 1.0

ShareDialog {
    id: page
    allowedOrientations: Orientation.All

    property string selectedCraneName: ""
    property double deletionTime: 1.0

    SailfishShare {
        id: shareItem
        source: page.source
        metadataStripped: true // What is this?
        serviceId: page.methodId
        userData: {
            //"accountId": root.accountId, // Apparently required in order for anything to work?
            //"scalePercent": root.scalePercent, // What is this?
            //"scalePercent": 1, // What is this?
            "description": ""+selectedCraneName+","+deletionTime,
        }
    }

    SilicaListView {
        id: listview
        anchors.fill: parent

        header: Component { Column {
            PageHeader {
                title: "Confirm Upload"
            }
            Label {
                text: "File: "+getNameFromPath(page.source)
            }
            Slider {
                id: timeSlider
                minimumValue: realToTimeSlider(10*60)
                maximumValue: 1.0
                value: 1.0
                valueText:
                    timeSliderToReal(value) < 120*60 ?
                        "" + Math.floor(timeSliderToReal(value) / 60) + " minutes" :
                    Math.floor(timeSliderToReal(value) / 3600) + " hours"
                label: "Deletion Time"
                x: Theme.paddingLarge
                width: page.width - Theme.paddingLarge * 2

                onValueChanged: {
                    deletionTime = timeSliderToReal(value)
                }
                Component.onCompleted: {
                    deletionTime = timeSliderToReal(value)
                }
            }
        }}

        VerticalScrollDecorator {}
        model: craneList
        delegate: ListItem {
            id: listitem
            height: Theme.itemSizeSmall
            highlighted: name === selectedCraneName
            Label {
                text: name
                anchors.verticalCenter: parent.verticalCenter
                //anchors.horizontalCenter: parent.horizontalCenter
                //anchors.left: parent.left + Theme.paddingLarge
                //anchors.margins: Theme.paddingLarge
                //x: Theme.paddingLarge
                anchors.left: parent.left
                anchors.margins: Theme.paddingLarge
                color: listitem.highlighted ?
                        Theme.highlightColor : Theme.primaryColor
            }
            onClicked: {
                console.log("listitem.onClicked")
                selectedCraneName = name
            }
        }

        ListModel {
            id: craneList
            Component.onCompleted: {
                refreshList()
            }
        }
    }

    onAccepted: {
        if(selectedCraneName === undefined || selectedCraneName === ""){
            console.log("onAccepted: No crane selected; not continuing")
            return
        }
        console.log("onAccepted: calling shareItem.start()")
        console.log("shareItem.userData: "+JSON.stringify(shareItem.userData))
        shareItem.start()
    }

    DBusInterface {
        id: metacrane_dbus

        service: "harbour.metacrane.client"
        iface: "harbour.metacrane.client"
        path: "/"
    }

    function refreshList(){
        metacrane_dbus.typedCall("getSavedCraneNames", [], function(result){
            console.log("Received saved crane names: "+JSON.stringify(result))
            craneList.clear()
            if(result.length == 0)
                result.push("foo");
            result.forEach(function(crane_name){
                if(selectedCraneName === undefined || selectedCraneName === "")
                    selectedCraneName = crane_name
                craneList.append({name: crane_name})
            })
        })
    }

    function timeSliderToReal(value){
        return Math.pow(value, 3) * 56*3600
    }
    function realToTimeSlider(seconds){
        return Math.pow(seconds/(56*3600), 1/3)
    }

    function getNameFromPath(path){
        var name = Qt.resolvedUrl(path).toString().match(/.*\/(.*)/)[1]
        if(name !== undefined)
            return name
        return path
    }
}
